@@ -13,7 +13,7 @@ except:
 import logging
 
 from lib.core.common import getSafeExString
-from lib.core.convert import utf8encode
+from lib.core.convert import getText
 from lib.core.data import conf
 from lib.core.data import logger
 from lib.core.exception import SqlmapConnectionException
@@ -75,7 +75,7 @@ class Connector(GenericConnector):
 
     def execute(self, query):
         try:
-            self.cursor.execute(utf8encode(query))
+            self.cursor.execute(getText(query))
         except self.__sqlite.OperationalError as ex:
             logger.log(logging.WARN if conf.dbmsHandler else logging.DEBUG, "(remote) '%s'" % getSafeExString(ex))
         except self.__sqlite.DatabaseError as ex:
