@@ -5,6 +5,8 @@ Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
+import re
+
 from lib.core.agent import agent
 from lib.core.common import arrayizeValue
 from lib.core.common import Backend
@@ -35,8 +37,9 @@ from lib.core.settings import METADB_SUFFIX
 from lib.request import inject
 from lib.utils.brute import columnExists
 from lib.utils.brute import tableExists
+from thirdparty import six
 
-class Search:
+class Search(object):
     """
     This class defines search functionalities for plugins.
     """
@@ -204,7 +207,7 @@ class Search:
                 if values and Backend.getIdentifiedDbms() in (DBMS.SQLITE, DBMS.FIREBIRD):
                     newValues = []
 
-                    if isinstance(values, basestring):
+                    if isinstance(values, six.string_types):
                         values = [values]
                     for value in values:
                         dbName = "SQLite" if Backend.isDbms(DBMS.SQLITE) else "Firebird"
@@ -375,7 +378,7 @@ class Search:
         colList = conf.col.split(',')
 
         if conf.exclude:
-            colList = [_ for _ in colList if _ not in conf.exclude.split(',')]
+            colList = [_ for _ in colList if re.search(conf.exclude, _, re.I) is None]
 
         origTbl = conf.tbl
         origDb = conf.db
