@@ -15,7 +15,7 @@ try:
     try:
         __import__("lib.utils.versioncheck")  # this has to be the first non-standard import
     except ImportError:
-        exit("[!] wrong installation detected (missing modules). Visit 'https://github.com/sqlmapproject/sqlmap/#installation' for further details")
+        sys.exit("[!] wrong installation detected (missing modules). Visit 'https://github.com/sqlmapproject/sqlmap/#installation' for further details")
 
     import bdb
     import distutils
@@ -27,47 +27,59 @@ try:
     import re
     import shutil
     import sys
-    import thread
+    import tempfile
     import threading
     import time
     import traceback
     import warnings
 
     warnings.filterwarnings(action="ignore", message=".*was already imported", category=UserWarning)
+    warnings.filterwarnings(action="ignore", message=".*using a very old release", category=UserWarning)
+    warnings.filterwarnings(action="ignore", message=".*default buffer size will be used", category=RuntimeWarning)
     warnings.filterwarnings(action="ignore", category=DeprecationWarning)
+    warnings.filterwarnings(action="ignore", category=UserWarning, module="psycopg2")
 
     from lib.core.data import logger
 
     from lib.core.common import banner
     from lib.core.common import checkIntegrity
+    from lib.core.common import checkPipedInput
     from lib.core.common import createGithubIssue
     from lib.core.common import dataToStdout
+    from lib.core.common import filterNone
+    from lib.core.common import getDaysFromLastUpdate
+    from lib.core.common import getFileItems
     from lib.core.common import getSafeExString
-    from lib.core.common import getUnicode
     from lib.core.common import maskSensitiveData
     from lib.core.common import openFile
     from lib.core.common import setPaths
     from lib.core.common import weAreFrozen
+    from lib.core.convert import getUnicode
     from lib.core.data import cmdLineOptions
     from lib.core.data import conf
     from lib.core.data import kb
-    from lib.core.common import unhandledExceptionMessage
     from lib.core.common import MKSTEMP_PREFIX
     from lib.core.common import setColor
+    from lib.core.common import unhandledExceptionMessage
+    from lib.core.compat import xrange
     from lib.core.exception import SqlmapBaseException
     from lib.core.exception import SqlmapShellQuitException
     from lib.core.exception import SqlmapSilentQuitException
     from lib.core.exception import SqlmapUserQuitException
-    from lib.core.option import initOptions
     from lib.core.option import init
+    from lib.core.option import initOptions
     from lib.core.patch import dirtyPatches
+    from lib.core.patch import resolveCrossReferences
     from lib.core.settings import GIT_PAGE
     from lib.core.settings import IS_WIN
+    from lib.core.settings import LAST_UPDATE_NAGGING_DAYS
     from lib.core.settings import LEGAL_DISCLAIMER
     from lib.core.settings import THREAD_FINALIZATION_TIMEOUT
     from lib.core.settings import UNICODE_ENCODING
     from lib.core.settings import VERSION
     from lib.parse.cmdline import cmdLineParser
+    from lib.utils.crawler import crawl
+    from thirdparty import six
 except KeyboardInterrupt:
     errMsg = "user aborted"
 
@@ -76,7 +88,7 @@ except KeyboardInterrupt:
         raise SystemExit
     else:
         import time
-        exit("\r[%s] [CRITICAL] %s" % (time.strftime("%X"), errMsg))
+        sys.exit("\r[%s] [CRITICAL] %s" % (time.strftime("%X"), errMsg))
 
 def modulePath():
     """
@@ -123,14 +135,19 @@ def main():
 
     try:
         dirtyPatches()
+        resolveCrossReferences()
         checkEnvironment()
         setPaths(modulePath())
         banner()
 
         # Store original command line options for possible later restoration
-        cmdLineOptions.update(cmdLineParser().__dict__)
+        args = cmdLineParser()
+        cmdLineOptions.update(args.__dict__ if hasattr(args, "__dict__") else args)
         initOptions(cmdLineOptions)
 
+        if checkPipedInput():
+            conf.batch = True
+
         if conf.get("api"):
             # heavy imports
             from lib.utils.api import StdDbOut
@@ -152,20 +169,50 @@ def main():
             # Postponed imports (faster start)
             if conf.smokeTest:
                 from lib.core.testing import smokeTest
-                smokeTest()
+                os._exitcode = 1 - (smokeTest() or 0)
+            elif conf.vulnTest:
+                from lib.core.testing import vulnTest
+                os._exitcode = 1 - (vulnTest() or 0)
             elif conf.liveTest:
                 from lib.core.testing import liveTest
-                liveTest()
+                os._exitcode = 1 - (liveTest() or 0)
             else:
                 from lib.controller.controller import start
-                if conf.profile:
+                if conf.profile and six.PY2:
                     from lib.core.profiling import profile
                     globals()["start"] = start
                     profile()
                 else:
                     try:
-                        start()
-                    except thread.error as ex:
+                        if conf.crawlDepth and conf.bulkFile:
+                            targets = getFileItems(conf.bulkFile)
+
+                            for i in xrange(len(targets)):
+                                try:
+                                    kb.targets.clear()
+                                    target = targets[i]
+
+                                    if not re.search(r"(?i)\Ahttp[s]*://", target):
+                                        target = "http://%s" % target
+
+                                    infoMsg = "starting crawler for target URL '%s' (%d/%d)" % (target, i + 1, len(targets))
+                                    logger.info(infoMsg)
+
+                                    crawl(target)
+                                except Exception as ex:
+                                    if not isinstance(ex, SqlmapUserQuitException):
+                                        errMsg = "problem occurred while crawling '%s' ('%s')" % (target, getSafeExString(ex))
+                                        logger.error(errMsg)
+                                    else:
+                                        raise
+                                else:
+                                    if kb.targets:
+                                        start()
+                        else:
+                            start()
+                    except Exception as ex:
+                        os._exitcode = 1
+
                         if "can't start new thread" in getSafeExString(ex):
                             errMsg = "unable to start new threads. Please check OS (u)limits"
                             logger.critical(errMsg)
@@ -223,12 +270,17 @@ def main():
             dataToStdout(excMsg)
             raise SystemExit
 
-        elif any(_ in excMsg for _ in ("ImportError", "Can't find file for module")):
+        elif any(_ in excMsg for _ in ("ImportError", "ModuleNotFoundError", "Can't find file for module")):
             errMsg = "invalid runtime environment ('%s')" % excMsg.split("Error: ")[-1].strip()
             logger.critical(errMsg)
             raise SystemExit
 
-        elif "MemoryError" in excMsg:
+        elif all(_ in excMsg for _ in ("SyntaxError: Non-ASCII character", ".py on line", "but no encoding declared")) or any(_ in excMsg for _ in ("source code string cannot contain null bytes", "No module named")):
+            errMsg = "invalid runtime environment ('%s')" % excMsg.split("Error: ")[-1].strip()
+            logger.critical(errMsg)
+            raise SystemExit
+
+        elif any(_ in excMsg for _ in ("MemoryError", "Cannot allocate memory")):
             errMsg = "memory exhaustion detected"
             logger.critical(errMsg)
             raise SystemExit
@@ -243,7 +295,12 @@ def main():
             logger.critical(errMsg)
             raise SystemExit
 
-        elif all(_ in excMsg for _ in ("No such file", "_'", "self.get_prog_name()")):
+        elif all(_ in excMsg for _ in ("Access is denied", "subprocess", "metasploit")):
+            errMsg = "permission error occurred while running Metasploit"
+            logger.critical(errMsg)
+            raise SystemExit
+
+        elif all(_ in excMsg for _ in ("No such file", "_'")):
             errMsg = "corrupted installation detected ('%s'). " % excMsg.strip().split('\n')[-1]
             errMsg += "You should retrieve the latest development version from official GitHub "
             errMsg += "repository at '%s'" % GIT_PAGE
@@ -265,11 +322,24 @@ def main():
             logger.critical(errMsg)
             raise SystemExit
 
+        elif "Invalid IPv6 URL" in excMsg:
+            errMsg = "invalid URL ('%s')" % excMsg.strip().split('\n')[-1]
+            logger.critical(errMsg)
+            raise SystemExit
+
         elif "_mkstemp_inner" in excMsg:
             errMsg = "there has been a problem while accessing temporary files"
             logger.critical(errMsg)
             raise SystemExit
 
+        elif any(_ in excMsg for _ in ("tempfile.mkdtemp", "tempfile.mkstemp")):
+            errMsg = "unable to write to the temporary directory '%s'. " % tempfile.gettempdir()
+            errMsg += "Please make sure that your disk is not full and "
+            errMsg += "that you have sufficient write permissions to "
+            errMsg += "create temporary files and/or directories"
+            logger.critical(errMsg)
+            raise SystemExit
+
         elif all(_ in excMsg for _ in ("twophase", "sqlalchemy")):
             errMsg = "please update the 'sqlalchemy' package (>= 1.1.11) "
             errMsg += "(Reference: https://qiita.com/tkprof/items/7d7b2d00df9c5f16fffe)"
@@ -284,7 +354,7 @@ def main():
 
         elif "must be pinned buffer, not bytearray" in excMsg:
             errMsg = "error occurred at Python interpreter which "
-            errMsg += "is fixed in 2.7.x. Please update accordingly "
+            errMsg += "is fixed in 2.7. Please update accordingly "
             errMsg += "(Reference: https://bugs.python.org/issue8104)"
             logger.critical(errMsg)
             raise SystemExit
@@ -297,6 +367,12 @@ def main():
             logger.critical(errMsg)
             raise SystemExit
 
+        elif "can't allocate read lock" in excMsg:
+            errMsg = "there has been a problem in regular socket operation "
+            errMsg += "('%s')" % excMsg.strip().split('\n')[-1]
+            logger.critical(errMsg)
+            raise SystemExit
+
         elif "'DictObject' object has no attribute '" in excMsg and all(_ in errMsg for _ in ("(fingerprinted)", "(identified)")):
             errMsg = "there has been a problem in enumeration. "
             errMsg += "Because of a considerable chance of false-positive case "
@@ -324,7 +400,10 @@ def main():
 
         for match in re.finditer(r'File "(.+?)", line', excMsg):
             file_ = match.group(1)
-            file_ = os.path.relpath(file_, os.path.dirname(__file__))
+            try:
+                file_ = os.path.relpath(file_, os.path.dirname(__file__))
+            except ValueError:
+                pass
             file_ = file_.replace("\\", '/')
             if "../" in file_:
                 file_ = re.sub(r"(\.\./)+", '/', file_)
@@ -346,6 +425,11 @@ def main():
     finally:
         kb.threadContinue = False
 
+        _ = getDaysFromLastUpdate()
+        if _ > LAST_UPDATE_NAGGING_DAYS:
+            warnMsg = "you haven't updated sqlmap for more than %d days!!!" % _
+            logger.warn(warnMsg)
+
         if conf.get("showTime"):
             dataToStdout("\n[*] ending @ %s\n\n" % time.strftime("%X /%Y-%m-%d/"), forceOutput=True)
 
@@ -358,15 +442,23 @@ def main():
                         os.remove(filepath)
                     except OSError:
                         pass
-            if not filter(None, (filepath for filepath in glob.glob(os.path.join(kb.tempDir, '*')) if not any(filepath.endswith(_) for _ in ('.lock', '.exe', '_')))):
-                shutil.rmtree(kb.tempDir, ignore_errors=True)
+
+            if not filterNone(filepath for filepath in glob.glob(os.path.join(kb.tempDir, '*')) if not any(filepath.endswith(_) for _ in (".lock", ".exe", ".so", '_'))):  # ignore junk files
+                try:
+                    shutil.rmtree(kb.tempDir, ignore_errors=True)
+                except OSError:
+                    pass
 
         if conf.get("hashDB"):
             conf.hashDB.flush(True)
 
         if conf.get("harFile"):
-            with openFile(conf.harFile, "w+b") as f:
-                json.dump(conf.httpCollector.obtain(), fp=f, indent=4, separators=(',', ': '))
+            try:
+                with openFile(conf.harFile, "w+b") as f:
+                    json.dump(conf.httpCollector.obtain(), fp=f, indent=4, separators=(',', ': '))
+            except SqlmapBaseException as ex:
+                errMsg = getSafeExString(ex)
+                logger.critical(errMsg)
 
         if conf.get("api"):
             conf.databaseCursor.disconnect()
@@ -391,10 +483,16 @@ if __name__ == "__main__":
         main()
     except KeyboardInterrupt:
         pass
+    except SystemExit:
+        raise
+    except:
+        traceback.print_exc()
     finally:
         # Reference: http://stackoverflow.com/questions/1635080/terminate-a-multi-thread-python-program
         if threading.activeCount() > 1:
-            os._exit(0)
+            os._exit(getattr(os, "_exitcode", 0))
+        else:
+            sys.exit(getattr(os, "_exitcode", 0))
 else:
     # cancelling postponed imports (because of Travis CI checks)
     from lib.controller.controller import start
